@@ -178,6 +178,7 @@ struct _ClockData {
 
         GtkWidget *showseconds_check;
         GtkWidget *showdate_check;
+        GtkWidget *showweeks_check;
         GtkWidget *custom_hbox;
         GtkWidget *custom_label;
         GtkWidget *custom_entry;
@@ -410,8 +411,8 @@ calculate_minimum_height (GtkWidget        *widget,
 #if GTK_CHECK_VERSION (3, 0, 0)
 	state = gtk_widget_get_state_flags (widget);
 	style_context = gtk_widget_get_style_context (widget);
-	font_desc = gtk_style_context_get_font (style_context, state);
 
+	gtk_style_context_get (style_context, state, GTK_STYLE_PROPERTY_FONT, &font_desc, NULL);
 	pango_context = gtk_widget_get_pango_context (widget);
 	metrics = pango_context_get_metrics (pango_context,
 					     font_desc,
@@ -877,6 +878,17 @@ create_calendar (ClockData *cd)
 	g_signal_connect (window, "key_press_event",
 			  G_CALLBACK (close_on_escape), cd->panel_button);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+        /*Name this window so the default theme can be overridden in panel theme,
+        otherwise default GtkWindow bg will be pulled in and override transparency */ 
+        gtk_widget_set_name(window, "MatePanelPopupWindow");
+
+        /* Make transparency possible in the theme */              
+        GdkScreen *screen = gtk_widget_get_screen(GTK_WIDGET(window));
+        GdkVisual *visual = gdk_screen_get_rgba_visual(screen);
+        gtk_widget_set_visual(GTK_WIDGET(window), visual);
+#endif
+
 	return window;
 }
 
@@ -3045,6 +3057,11 @@ fill_prefs_window (ClockData *cd)
 	g_settings_bind (cd->settings, KEY_SHOW_SECONDS, widget, "active",
                          G_SETTINGS_BIND_DEFAULT);
 
+        /* Set the "Show Week Numbers" checkbox */
+	widget = _clock_get_widget (cd, "weeks_check");
+	g_settings_bind (cd->settings, KEY_SHOW_WEEK, widget, "active",
+	G_SETTINGS_BIND_DEFAULT);
+	 
 	/* Set the "Show weather" checkbox */
 	widget = _clock_get_widget (cd, "weather_check");
 	g_settings_bind (cd->settings, KEY_SHOW_WEATHER, widget, "active",
