@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  *
  * Authors:
  *      Vincent Untz <vuntz@gnome.org>
@@ -32,7 +32,7 @@ extern "C" {
 #endif
 
 #define CLOCK_ICON "mate-panel-clock"
-#define CLOCK_SCHEMA_DIR "/schemas/apps/clock_applet/prefs"
+#define CLOCK_SCHEMA "org.mate.panel.applet.clock"
 
 #ifdef __cplusplus
 }
