@@ -26,6 +26,10 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+#include <gtk/gtkx.h>
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
