@@ -261,6 +261,7 @@ tray_removed (NaTrayManager *manager,
     return;
 
   priv = tray->priv;
+
   g_assert (tray->priv->trays_screen == trays_screen);
 
   gtk_container_remove (GTK_CONTAINER (priv->box), icon);
@@ -545,7 +546,7 @@ na_tray_expose_icon (GtkWidget *widget,
 #endif
 		     gpointer   data)
 {
-  cairo_t *cr = data;
+  cairo_t *cr = (cairo_t *) data;
 
   if (na_tray_child_has_alpha (NA_TRAY_CHILD (widget)))
     {
@@ -895,6 +896,39 @@ idle_redraw_cb (NaTray *tray)
   return FALSE;
 }
 
+void
+na_tray_set_padding (NaTray *tray,
+                     gint    padding)
+{
+  NaTrayPrivate *priv = tray->priv;
+
+  if (get_tray (priv->trays_screen) == tray)
+    na_tray_manager_set_padding (priv->trays_screen->tray_manager, padding);
+}
+
+void
+na_tray_set_icon_size (NaTray *tray,
+                       gint    size)
+{
+  NaTrayPrivate *priv = tray->priv;
+
+  if (get_tray (priv->trays_screen) == tray)
+    na_tray_manager_set_icon_size (priv->trays_screen->tray_manager, size);
+}
+
+void
+na_tray_set_colors (NaTray   *tray,
+                    GdkColor *fg,
+                    GdkColor *error,
+                    GdkColor *warning,
+                    GdkColor *success)
+{
+  NaTrayPrivate *priv = tray->priv;
+
+  if (get_tray (priv->trays_screen) == tray)
+    na_tray_manager_set_colors (priv->trays_screen->tray_manager, fg, error, warning, success);
+}
+
 void
 na_tray_force_redraw (NaTray *tray)
 {
