@@ -33,6 +33,8 @@
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE
 #include <libwnck/libwnck.h>
 
+#include <libmate-desktop/mate-aboutdialog.h>
+
 #include "wncklet.h"
 #include "showdesktop.h"
 
@@ -130,7 +132,13 @@ static void button_size_allocated(GtkWidget* button, GtkAllocation* allocation,
 
 static void update_icon(ShowDesktopData* sdd)
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GtkStyleContext *context;
+	GtkStateFlags    state;
+	GtkBorder        padding;
+#else
 	GtkStyle* style;
+#endif
 	int width, height;
 	GdkPixbuf* icon;
 	GdkPixbuf* scaled;
@@ -143,6 +151,26 @@ static void update_icon(ShowDesktopData* sdd)
 	if (!sdd->icon_theme)
 		return;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	state = gtk_widget_get_state_flags (sdd->button);
+	context = gtk_widget_get_style_context (sdd->button);
+	gtk_style_context_get_padding (context, state, &padding);
+	gtk_style_context_get_style (context,
+			             "focus-line-width", &focus_width,
+			             "focus-padding", &focus_pad,
+			             NULL);
+
+	switch (sdd->orient) {
+	case GTK_ORIENTATION_HORIZONTAL:
+		thickness = padding.top + padding.bottom;
+		break;
+	case GTK_ORIENTATION_VERTICAL:
+		thickness = padding.left + padding.right;
+		break;
+	}
+
+	icon_size = sdd->size - 2 * (focus_width + focus_pad) - thickness;
+#else
 	gtk_widget_style_get (sdd->button, "focus-line-width", &focus_width, "focus-padding", &focus_pad, NULL);
 
 	style = gtk_widget_get_style(sdd->button);
@@ -157,7 +185,8 @@ static void update_icon(ShowDesktopData* sdd)
 			break;
 	}
 
-	icon_size = sdd->size - 2 * (focus_width + focus_pad + thickness);
+	icon_size = sdd->size - 2 * (focus_width + focus_pad) - thickness;
+#endif
 
 	if (icon_size < 22)
 		icon_size = 16;
@@ -375,6 +404,9 @@ gboolean show_desktop_applet_fill(MatePanelApplet* applet)
 	GtkActionGroup* action_group;
 	gchar* ui_path;
 	AtkObject* atk_obj;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GtkCssProvider *provider;
+#endif
 
 	mate_panel_applet_set_flags(applet, MATE_PANEL_APPLET_EXPAND_MINOR);
 
@@ -404,6 +436,19 @@ gboolean show_desktop_applet_fill(MatePanelApplet* applet)
 	sdd->button = gtk_toggle_button_new ();
 
 	gtk_widget_set_name (sdd->button, "showdesktop-button");
+#if GTK_CHECK_VERSION (3, 0, 0)
+	provider = gtk_css_provider_new ();
+	gtk_css_provider_load_from_data (provider,
+					 "#showdesktop-button {\n"
+					 " -GtkWidget-focus-line-width: 0px;\n"
+					 " -GtkWidget-focus-padding: 0px; }",
+					 -1, NULL);
+
+	gtk_style_context_add_provider (gtk_widget_get_style_context (sdd->button),
+					GTK_STYLE_PROVIDER (provider),
+					GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+					g_object_unref (provider);
+#else
 	gtk_rc_parse_string ("\n"
 		"   style \"showdesktop-button-style\"\n"
 		"   {\n"
@@ -413,6 +458,7 @@ gboolean show_desktop_applet_fill(MatePanelApplet* applet)
 		"\n"
 		"    widget \"*.showdesktop-button\" style \"showdesktop-button-style\"\n"
 		"\n");
+#endif
 
 	atk_obj = gtk_widget_get_accessible(sdd->button);
 	atk_object_set_name (atk_obj, _("Show Desktop Button"));
@@ -479,7 +525,7 @@ static void display_about_dialog(GtkAction* action, ShowDesktopData* sdd)
 		"Copyright \xc2\xa9 2011 Perberos\n"
 		"Copyright \xc2\xa9 2002 Red Hat, Inc.";
 
-	gtk_show_about_dialog(GTK_WINDOW(sdd->applet),
+	mate_show_about_dialog(GTK_WINDOW(sdd->applet),
 		"program-name", _("Show Desktop Button"),
 		"authors", authors,
 		"comments", _("This button lets you hide all windows and show the desktop."),
