@@ -25,6 +25,7 @@
 #include <libwnck/libwnck.h>
 #include <gio/gio.h>
 
+#include <libmate-desktop/mate-aboutdialog.h>
 #include <libmate-desktop/mate-gsettings.h>
 
 #include "workspace-switcher.h"
@@ -40,7 +41,7 @@
 #define NEVER_SENSITIVE "never_sensitive"
 #define MARCO_GENERAL_SCHEMA "org.mate.Marco.general"
 #define NUM_WORKSPACES "num-workspaces"
-#define MARCO_WORSKACES_SCHEMA "org.mate.Marco.workspace-names"
+#define MARCO_WORKSPACES_SCHEMA "org.mate.Marco.workspace-names"
 #define WORKSPACE_NAME "name-1"
 
 #define WORKSPACE_SWITCHER_ICON "mate-panel-workspace-switcher"
@@ -143,7 +144,7 @@ static void window_manager_changed(WnckScreen* screen, PagerData* pager)
 
 	if (!wm_name)
 		pager->wm = PAGER_WM_UNKNOWN;
-	else if (strcmp(wm_name, "Marco") == 0)
+	else if (strcmp(wm_name, "Metacity (Marco)") == 0)
 		pager->wm = PAGER_WM_MARCO;
 	else if (strcmp(wm_name, "Compiz") == 0)
 		pager->wm = PAGER_WM_COMPIZ;
@@ -488,6 +489,9 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 	GtkActionGroup* action_group;
 	gchar* ui_path;
 	gboolean display_names;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GtkCssProvider *provider;
+#endif
 
 	pager = g_new0(PagerData, 1);
 
@@ -530,7 +534,7 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 	}
 
 #ifdef WNCK_CHECK_VERSION
-#if WNCK_CHECK_VERSION (2, 91, 6)
+#if WNCK_CHECK_VERSION (3, 0, 0)
 	pager->pager = wnck_pager_new();
 #else
 	pager->pager = wnck_pager_new(NULL);
@@ -542,6 +546,17 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 	pager->wm = PAGER_WM_UNKNOWN;
 	wnck_pager_set_shadow_type(WNCK_PAGER(pager->pager), GTK_SHADOW_IN);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	provider = gtk_css_provider_new ();
+	gtk_css_provider_load_from_data (provider,
+                                         "WnckPager:selected {\n"
+                                         "background-color: #4A90D9; }",
+                                         -1, NULL);
+	gtk_style_context_add_provider (gtk_widget_get_style_context (pager->pager),
+					GTK_STYLE_PROVIDER (provider),
+					GTK_STYLE_PROVIDER_PRIORITY_FALLBACK);
+	g_object_unref (provider);
+#endif
 	g_signal_connect(G_OBJECT(pager->pager), "destroy", G_CALLBACK(destroy_pager), pager);
 
 	gtk_container_add(GTK_CONTAINER(pager->applet), pager->pager);
@@ -603,7 +618,7 @@ static void display_about_dialog(GtkAction* action, PagerData* pager)
 		"Copyright \xc2\xa9 2011 Perberos\n"
 		"Copyright \xc2\xa9 2002 Red Hat, Inc.";
 
-	gtk_show_about_dialog(GTK_WINDOW(pager->applet),
+	mate_show_about_dialog(GTK_WINDOW(pager->applet),
 		"program-name", _("Workspace Switcher"),
 		"authors", authors,
 		"comments", _("The Workspace Switcher shows you a small version of your workspaces that lets you manage your windows."),
@@ -841,8 +856,8 @@ static void setup_dialog(GtkBuilder* builder, PagerData* pager)
 
 	if (mate_gsettings_schema_exists(MARCO_GENERAL_SCHEMA))
 		marco_general_settings = g_settings_new (MARCO_GENERAL_SCHEMA);
-	if (mate_gsettings_schema_exists(MARCO_WORSKACES_SCHEMA))
-		marco_workspaces_settings = g_settings_new (MARCO_WORSKACES_SCHEMA);
+	if (mate_gsettings_schema_exists(MARCO_WORKSPACES_SCHEMA))
+		marco_workspaces_settings = g_settings_new (MARCO_WORKSPACES_SCHEMA);
 
 	pager->workspaces_frame = WID("workspaces_frame");
 	pager->workspace_names_label = WID("workspace_names_label");
