@@ -41,6 +41,8 @@ static GSList *registered_applets = NULL;
 static GSList *queued_position_saves = NULL;
 static guint   queued_position_source = 0;
 
+static void applet_menu_show (GtkWidget *w, AppletInfo *info);
+static void applet_menu_deactivate (GtkWidget *w, AppletInfo *info);
 
 static inline PanelWidget *
 mate_panel_applet_get_panel_widget (AppletInfo *info)
@@ -65,8 +67,6 @@ mate_panel_applet_set_dnd_enabled (AppletInfo *info,
 		break;
 	case PANEL_OBJECT_APPLET:
 		break;
-	case PANEL_OBJECT_LOGOUT:
-	case PANEL_OBJECT_LOCK:
 	case PANEL_OBJECT_ACTION:
 		panel_action_button_set_dnd_enabled (PANEL_ACTION_BUTTON (info->widget),
 						     dnd_enabled);
@@ -166,6 +166,9 @@ mate_panel_applet_recreate_menu (AppletInfo	*info)
 		menu->submenu =NULL;
 	}
 
+	g_signal_handlers_disconnect_by_func (info->menu, G_CALLBACK (applet_menu_show), info);
+	g_signal_handlers_disconnect_by_func (info->menu, G_CALLBACK (applet_menu_deactivate), info);
+
 	g_object_unref (info->menu);
 	info->menu = mate_panel_applet_create_menu (info);
 }
@@ -263,8 +266,6 @@ applet_callback_callback (GtkWidget      *widget,
 			PANEL_MENU_BUTTON (menu->info->widget), menu->name);
 		break;
 	case PANEL_OBJECT_ACTION:
-	case PANEL_OBJECT_LOGOUT:
-	case PANEL_OBJECT_LOCK:
 		panel_action_button_invoke_menu (
 			PANEL_ACTION_BUTTON (menu->info->widget), menu->name);
 		break;
@@ -563,6 +564,19 @@ mate_panel_applet_create_menu (AppletInfo *info)
 		return NULL;
 	}
 
+/* Set up theme and transparency support */
+#if GTK_CHECK_VERSION (3, 0, 0) 
+	GtkWidget *toplevel = gtk_widget_get_toplevel (menu);
+/* Fix any failures of compiz/other wm's to communicate with gtk for transparency */
+	GdkScreen *screen = gtk_widget_get_screen(GTK_WIDGET(toplevel));
+	GdkVisual *visual = gdk_screen_get_rgba_visual(screen);
+	gtk_widget_set_visual(GTK_WIDGET(toplevel), visual); 
+/* Set menu and it's toplevel window to follow panel theme */
+	GtkStyleContext *context;
+	context = gtk_widget_get_style_context (GTK_WIDGET(toplevel));
+	gtk_style_context_add_class(context,"gnome-panel-menu-bar");
+	gtk_style_context_add_class(context,"mate-panel-menu-bar");
+#endif
 	return menu;
 }
 
@@ -598,6 +612,9 @@ mate_panel_applet_position_menu (GtkMenu   *menu,
 {
 	GtkAllocation   allocation;
 	GtkRequisition  requisition;
+#if GTK_CHECK_VERSION(3, 0, 0)
+	GdkDevice      *device;
+#endif
 	GdkScreen      *screen;
 	GtkWidget      *parent;
 	int             menu_x = 0;
@@ -618,8 +635,12 @@ mate_panel_applet_position_menu (GtkMenu   *menu,
 #endif
 
 	gdk_window_get_origin (gtk_widget_get_window (applet), &menu_x, &menu_y);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	device = gdk_device_manager_get_client_pointer (gdk_display_get_device_manager (gtk_widget_get_display (applet)));
+	gdk_window_get_device_position(gtk_widget_get_window (applet), device, &pointer_x, &pointer_y, NULL);
+#else
 	gtk_widget_get_pointer (applet, &pointer_x, &pointer_y);
-
+#endif
 	gtk_widget_get_allocation (applet, &allocation);
 
 	if (!gtk_widget_get_has_window (applet)) {
@@ -661,7 +682,11 @@ mate_panel_applet_position_menu (GtkMenu   *menu,
 
 	*x = menu_x;
 	*y = menu_y;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	*push_in = FALSE;
+#else
 	*push_in = TRUE;
+#endif
 }
 
 static void
@@ -777,8 +802,11 @@ mate_panel_applet_destroy (GtkWidget  *widget,
 		panel_lockdown_notify_remove (G_CALLBACK (mate_panel_applet_recreate_menu),
 					      info);
 
-	if (info->menu)
+	if (info->menu) {
+		g_signal_handlers_disconnect_by_func (info->menu, G_CALLBACK (applet_menu_show), info);
+		g_signal_handlers_disconnect_by_func (info->menu, G_CALLBACK (applet_menu_deactivate), info);
 		g_object_unref (info->menu);
+	}
 	info->menu = NULL;
 
 	if (info->data_destroy)
@@ -1171,6 +1199,17 @@ mate_panel_applet_save_position (AppletInfo *applet_info,
 	   "position" properties of an applet that may in fact be locked down.
 	   So check if these are writable before attempting to write them */
 
+	locked = panel_widget_get_applet_locked (panel_widget, applet_info->widget) ? 1 : 0;
+	if (g_settings_get_boolean (applet_info->settings, PANEL_OBJECT_LOCKED_KEY) ? 1 : 0 != locked)
+		g_settings_set_boolean (applet_info->settings, PANEL_OBJECT_LOCKED_KEY, locked);
+
+	if (locked) {
+		// Until position calculations are refactored to fix the issue of the panel applets
+		// getting reordered on resolution changes...
+		// .. don't save position/right-stick on locked applets
+		return;
+	}
+
 	right_stick = panel_is_applet_right_stick (applet_info->widget) ? 1 : 0;
 	if (g_settings_is_writable (applet_info->settings, PANEL_OBJECT_PANEL_RIGHT_STICK_KEY) &&
 	    (g_settings_get_boolean (applet_info->settings, PANEL_OBJECT_PANEL_RIGHT_STICK_KEY) ? 1 : 0) != right_stick)
@@ -1183,10 +1222,6 @@ mate_panel_applet_save_position (AppletInfo *applet_info,
 	if (g_settings_is_writable (applet_info->settings, PANEL_OBJECT_POSITION_KEY) &&
 	    g_settings_get_int (applet_info->settings, PANEL_OBJECT_POSITION_KEY) != position)
 		g_settings_set_int (applet_info->settings, PANEL_OBJECT_POSITION_KEY, position);
-
-	locked = panel_widget_get_applet_locked (panel_widget, applet_info->widget) ? 1 : 0;
-	if (g_settings_get_boolean (applet_info->settings, PANEL_OBJECT_LOCKED_KEY) ? 1 : 0 != locked)
-		g_settings_set_boolean (applet_info->settings, PANEL_OBJECT_LOCKED_KEY, locked);
 }
 
 const char *
