@@ -18,9 +18,6 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#if GTK_CHECK_VERSION (3, 0, 0)
-#include <gdk/gdkkeysyms-compat.h>
-#endif
 #include <gio/gio.h>
 
 #include "drawer.h"
@@ -50,20 +47,26 @@ toplevel_destroyed (GtkWidget *widget,
 		    Drawer    *drawer)
 {
 	drawer->toplevel = NULL;
-	gtk_widget_destroy (drawer->button);
+
+	if (drawer->button) {
+		gtk_widget_destroy (drawer->button);
+		drawer->button = NULL;
+	}
 }
 
 static void
 destroy_drawer (GtkWidget *widget,
 		Drawer    *drawer)
 {
-	if (drawer->toplevel)
+	if (drawer->toplevel) {
 		gtk_widget_destroy (GTK_WIDGET (drawer->toplevel));
-	drawer->toplevel = NULL;
+		drawer->toplevel = NULL;
+	}
 
-	if (drawer->close_timeout_id)
+	if (drawer->close_timeout_id) {
 		g_source_remove (drawer->close_timeout_id);
-	drawer->close_timeout_id = 0;
+		drawer->close_timeout_id = 0;
+	}
 }
 
 static void
@@ -93,8 +96,8 @@ key_press_drawer (GtkWidget   *widget,
 	orient = PANEL_WIDGET (gtk_widget_get_parent (drawer->button))->orient;
 
 	switch (event->keyval) {
-	case GDK_Up:
-	case GDK_KP_Up:
+	case GDK_KEY_Up:
+	case GDK_KEY_KP_Up:
 		if (orient == GTK_ORIENTATION_HORIZONTAL) {
 			if (!panel_toplevel_get_is_hidden (drawer->toplevel))
 				drawer_focus_panel_widget (drawer, GTK_DIR_TAB_BACKWARD);
@@ -103,8 +106,8 @@ key_press_drawer (GtkWidget   *widget,
 			retval = FALSE;
 		}
 		break;
-	case GDK_Left:
-	case GDK_KP_Left:
+	case GDK_KEY_Left:
+	case GDK_KEY_KP_Left:
 		if (orient == GTK_ORIENTATION_VERTICAL) {
 			if (!panel_toplevel_get_is_hidden (drawer->toplevel))
 				drawer_focus_panel_widget (drawer, GTK_DIR_TAB_BACKWARD);
@@ -113,8 +116,8 @@ key_press_drawer (GtkWidget   *widget,
 			retval = FALSE;
 		}
 		break;
-	case GDK_Down:
-	case GDK_KP_Down:
+	case GDK_KEY_Down:
+	case GDK_KEY_KP_Down:
 		if (orient == GTK_ORIENTATION_HORIZONTAL) {
 			if (!panel_toplevel_get_is_hidden (drawer->toplevel))
 				drawer_focus_panel_widget (drawer, GTK_DIR_TAB_FORWARD);
@@ -123,8 +126,8 @@ key_press_drawer (GtkWidget   *widget,
 			retval = FALSE;
 		}
 		break;
-	case GDK_Right:
-	case GDK_KP_Right:
+	case GDK_KEY_Right:
+	case GDK_KEY_KP_Right:
 		if (orient == GTK_ORIENTATION_VERTICAL) {
 			if (!panel_toplevel_get_is_hidden (drawer->toplevel))
 				drawer_focus_panel_widget (drawer, GTK_DIR_TAB_FORWARD);
@@ -133,7 +136,7 @@ key_press_drawer (GtkWidget   *widget,
 			retval = FALSE;
 		}
 		break;
-	case GDK_Escape:
+	case GDK_KEY_Escape:
 		panel_toplevel_hide (drawer->toplevel, FALSE, -1);
 		break;
 	default:
@@ -156,7 +159,7 @@ key_press_drawer_widget (GtkWidget   *widget,
 {
 	PanelWidget *panel_widget;
 
-	if (event->keyval != GDK_Escape)
+	if (event->keyval != GDK_KEY_Escape)
 		return FALSE;
 
 	panel_widget = panel_toplevel_get_panel_widget (drawer->toplevel);
@@ -305,13 +308,11 @@ set_tooltip_and_name (Drawer     *drawer,
 {
 	g_return_if_fail (drawer != NULL);
 	g_return_if_fail (drawer->toplevel != NULL);
-	g_return_if_fail (tooltip != NULL);
 
-	if (tooltip && !tooltip [0])
-		tooltip = NULL;
-
-	panel_toplevel_set_name (drawer->toplevel, tooltip);
-	panel_util_set_tooltip_text (drawer->button, tooltip);
+	if (tooltip != NULL && tooltip [0] != '\0') {
+		panel_toplevel_set_name (drawer->toplevel, tooltip);
+		panel_util_set_tooltip_text (drawer->button, tooltip);
+	}
 }
 
 static Drawer *
@@ -416,37 +417,20 @@ drawer_button_size_allocated (GtkWidget     *widget,
 }
 
 static void
-panel_drawer_use_custom_icon_changed (GSettings *settings,
+panel_drawer_custom_icon_changed (GSettings *settings,
 				      gchar     *key,
 				      Drawer      *drawer)
 {
-	gboolean  use_custom_icon;
-	char     *custom_icon = NULL;
+	g_return_if_fail (drawer != NULL);
+	g_return_if_fail (drawer->button != NULL);
 
-	use_custom_icon = g_settings_get_boolean (settings, key);
+	gboolean use_custom_icon = g_settings_get_boolean (settings, PANEL_OBJECT_USE_CUSTOM_ICON_KEY);
+	char *custom_icon = g_settings_get_string (settings, PANEL_OBJECT_CUSTOM_ICON_KEY);
 
-	if (use_custom_icon) {
-		custom_icon = g_settings_get_string (settings, PANEL_OBJECT_CUSTOM_ICON_KEY);
-	}
-
-	button_widget_set_icon_name (BUTTON_WIDGET (drawer->button), custom_icon);
-
-	g_free (custom_icon);
-}
-
-static void
-panel_drawer_custom_icon_changed (GSettings *settings,
-				  gchar     *key,
-				  Drawer    *drawer)
-{
-	char *custom_icon;
-	custom_icon = g_settings_get_string (settings, key);
-
-	if (custom_icon && custom_icon [0]) {
-		gboolean    use_custom_icon;
-		use_custom_icon = g_settings_get_boolean (settings, PANEL_OBJECT_USE_CUSTOM_ICON_KEY);
-		if (use_custom_icon)
-			button_widget_set_icon_name (BUTTON_WIDGET (drawer->button), custom_icon);
+	if (use_custom_icon && custom_icon != NULL && custom_icon [0] != '\0') {
+		button_widget_set_icon_name (BUTTON_WIDGET (drawer->button), custom_icon);
+	} else {
+		button_widget_set_icon_name (BUTTON_WIDGET (drawer->button), PANEL_ICON_DRAWER);
 	}
 
 	g_free (custom_icon);
@@ -467,7 +451,7 @@ panel_drawer_connect_to_gsettings (Drawer *drawer)
 {
 	g_signal_connect (drawer->info->settings,
 			  "changed::" PANEL_OBJECT_USE_CUSTOM_ICON_KEY,
-			  G_CALLBACK (panel_drawer_use_custom_icon_changed),
+			  G_CALLBACK (panel_drawer_custom_icon_changed),
 			  drawer);
 	g_signal_connect (drawer->info->settings,
 			  "changed::" PANEL_OBJECT_CUSTOM_ICON_KEY,
