@@ -109,7 +109,12 @@ panel_app_info_launch_uris (GAppInfo   *appinfo,
 	g_return_val_if_fail (GDK_IS_SCREEN (screen), FALSE);
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+	GdkDisplay *display = gdk_display_get_default ();
+	context = gdk_display_get_app_launch_context (display);
+#else
 	context = gdk_app_launch_context_new ();
+#endif
 	gdk_app_launch_context_set_screen (context, screen);
 	gdk_app_launch_context_set_timestamp (context, timestamp);
 
