@@ -30,6 +30,8 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 
+#include <libmate-desktop/mate-gsettings.h>
+
 #include "panel-schemas.h"
 #include "panel-profile.h"
 #include "panel-xutils.h"
@@ -130,7 +132,9 @@ panel_binding_changed (GSettings *settings,
 	binding->keyval    = 0;
 	binding->modifiers = 0;
 
-	panel_binding_set_from_string (binding, g_settings_get_string (settings, key));
+	gchar *value = g_settings_get_string (settings, key);
+	panel_binding_set_from_string (binding, value);
+	g_free (value);
 
 	if (!binding->keyval)
 		return;
@@ -175,7 +179,9 @@ panel_bindings_mouse_modifier_changed (GSettings *settings,
 									   gchar *key,
 									   gpointer user_data)
 {
-	panel_bindings_mouse_modifier_set_from_string (g_settings_get_string (settings, key));
+	gchar *value = g_settings_get_string (settings, key);
+	panel_bindings_mouse_modifier_set_from_string (value);
+	g_free (value);
 }
 
 static void
@@ -187,6 +193,11 @@ panel_bindings_initialise (void)
 	if (initialised)
 		return;
 
+	if (!mate_gsettings_schema_exists (MARCO_SCHEMA)) {
+		initialised = TRUE;
+		return;
+	}
+
 	marco_settings = g_settings_new (MARCO_SCHEMA);
 	marco_keybindings_settings = g_settings_new (MARCO_KEYBINDINGS_SCHEMA);
 
