@@ -32,9 +32,15 @@
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 
+#include <X11/extensions/XInput2.h>
+
 #include "panel-icon-names.h"
 #include "panel-stock-icons.h"
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+#define gtk_vbox_new(X, Y) gtk_box_new(GTK_ORIENTATION_VERTICAL, Y)
+#endif
+
 static GdkFilterReturn popup_filter (GdkXEvent *gdk_xevent,
 				     GdkEvent  *event,
 				     GtkWidget *popup);
@@ -100,15 +106,32 @@ display_popup_window (GdkScreen *screen)
 static void
 remove_popup (GtkWidget *popup)
 {
-	GdkWindow *root;
+	GdkWindow        *root;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GdkDisplay       *display;
+	GdkDevice        *pointer;
+	GdkDevice        *keyboard;
+	GdkDeviceManager *device_manager;
+#endif
 
 	root = gdk_screen_get_root_window (
 			gtk_window_get_screen (GTK_WINDOW (popup)));
 	gdk_window_remove_filter (root, (GdkFilterFunc) popup_filter, popup);
 
 	gtk_widget_destroy (popup);
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+	display = gdk_window_get_display (root);
+	device_manager = gdk_display_get_device_manager (display);
+	pointer = gdk_device_manager_get_client_pointer (device_manager);
+	keyboard = gdk_device_get_associated_device (pointer);
+
+	gdk_device_ungrab (pointer, GDK_CURRENT_TIME);
+	gdk_device_ungrab (keyboard, GDK_CURRENT_TIME);
+#else
 	gdk_pointer_ungrab (GDK_CURRENT_TIME);
 	gdk_keyboard_ungrab (GDK_CURRENT_TIME);
+#endif
 }
 
 static gboolean
@@ -240,26 +263,23 @@ kill_window_question (gpointer window)
 
 static void 
 handle_button_press_event (GtkWidget *popup,
-			   XKeyEvent *event)
+			   Display *display,
+			   Window subwindow)
 {
 	Window window;
 
 	remove_popup (popup);
 
-	if (event->subwindow == None)
+	if (subwindow == None)
 		return;
 
 	if (wm_state_atom == None)
-		wm_state_atom = XInternAtom (event->display, "WM_STATE", FALSE);
+		wm_state_atom = XInternAtom (display, "WM_STATE", FALSE);
 
-	window = find_managed_window (event->display, event->subwindow);
+	window = find_managed_window (display, subwindow);
 
 	if (window != None) {
-#if GTK_CHECK_VERSION (3, 0, 0)
-		if (!gdk_x11_window_lookup_for_display (gdk_x11_lookup_xdisplay (event->display), window))
-#else
-		if (!gdk_xid_table_lookup_for_display (gdk_x11_lookup_xdisplay (event->display), window))
-#endif
+		if (!gdk_x11_window_lookup_for_display (gdk_x11_lookup_xdisplay (display), window))
 			kill_window_question ((gpointer) window);
 	}
 }
@@ -270,10 +290,12 @@ popup_filter (GdkXEvent *gdk_xevent,
 	      GtkWidget *popup)
 {
 	XEvent *xevent = (XEvent *) gdk_xevent;
+	XIEvent *xiev;
+	XIDeviceEvent *xidev;
 
 	switch (xevent->type) {
 	case ButtonPress:
-		handle_button_press_event (popup, &xevent->xkey);
+		handle_button_press_event (popup, xevent->xbutton.display, xevent->xbutton.subwindow);
 		return GDK_FILTER_REMOVE;
 	case KeyPress:
 		if (xevent->xkey.keycode == XKeysymToKeycode (xevent->xany.display, XK_Escape)) {
@@ -281,6 +303,21 @@ popup_filter (GdkXEvent *gdk_xevent,
 			return GDK_FILTER_REMOVE;
 		}
 		break;
+	case GenericEvent:
+		xiev = (XIEvent *) xevent->xcookie.data;
+		xidev = (XIDeviceEvent *) xiev;
+		switch (xiev->evtype) {
+		case XI_KeyPress:
+			if (xidev->detail == XKeysymToKeycode (xevent->xany.display, XK_Escape)) {
+				remove_popup (popup);
+				return GDK_FILTER_REMOVE;
+			}
+			break;
+		case XI_ButtonPress:
+			handle_button_press_event (popup, xidev->display, xidev->child);
+			return GDK_FILTER_REMOVE;
+		}
+		break;
 	default:
 		break;
 	}
@@ -296,6 +333,12 @@ panel_force_quit (GdkScreen *screen,
 	GdkCursor     *cross;
 	GtkWidget     *popup;
 	GdkWindow     *root;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GdkDisplay *display;
+	GdkDevice *pointer;
+	GdkDevice *keyboard;
+	GdkDeviceManager *device_manager;
+#endif
 
 	popup = display_popup_window (screen);
 
@@ -304,9 +347,35 @@ panel_force_quit (GdkScreen *screen,
 	gdk_window_add_filter (root, (GdkFilterFunc) popup_filter, popup);
 
 	cross = gdk_cursor_new (GDK_CROSS);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	display = gdk_window_get_display (root);
+	device_manager = gdk_display_get_device_manager (display);
+	pointer = gdk_device_manager_get_client_pointer (device_manager);
+	keyboard = gdk_device_get_associated_device (pointer);
+
+	status = gdk_device_grab (pointer, root,
+				  GDK_OWNERSHIP_NONE, FALSE,
+				  GDK_BUTTON_PRESS_MASK,
+				  cross, time);
+
+	g_object_unref (cross);
+
+	status = gdk_device_grab (keyboard, root,
+				  GDK_OWNERSHIP_NONE, FALSE,
+				  GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK,
+				  NULL, time);
+
+	if (status != GDK_GRAB_SUCCESS) {
+		g_warning ("Pointer grab failed\n");
+		remove_popup (popup);
+		return;
+	}
+#else
 	status = gdk_pointer_grab (root, FALSE, GDK_BUTTON_PRESS_MASK,
 				   NULL, cross, time);
+
 	gdk_cursor_unref (cross);
+
 	if (status != GDK_GRAB_SUCCESS) {
 		g_warning ("Pointer grab failed\n");
 		remove_popup (popup);
@@ -319,6 +388,6 @@ panel_force_quit (GdkScreen *screen,
 		remove_popup (popup);
 		return;
 	}
-
+#endif
 	gdk_flush ();
 }
