@@ -87,14 +87,12 @@ typedef struct {
 	GList		 *completion_items;
 	GCompletion      *completion;
 
-	GSList           *add_icon_paths;
-	int	          add_icons_idle_id;
 	int	          add_items_idle_id;
 	int		  find_command_idle_id;
 	gboolean	  use_program_list;
 	gboolean	  completion_started;
 
-	char		 *icon_path;
+	GIcon		 *icon;
 	char             *desktop_path;
 	char		 *item_name;
 
@@ -102,8 +100,7 @@ typedef struct {
 } PanelRunDialog;
 
 enum {
-	COLUMN_ICON,
-	COLUMN_ICON_FILE,
+	COLUMN_GICON,
 	COLUMN_NAME,
 	COLUMN_COMMENT,
 	COLUMN_PATH,
@@ -188,21 +185,12 @@ panel_run_dialog_destroy (PanelRunDialog *dialog)
 
 	g_object_unref (dialog->program_list_box);
 
-	g_slist_foreach (dialog->add_icon_paths, (GFunc) gtk_tree_path_free, NULL);
-	g_slist_free (dialog->add_icon_paths);
-	dialog->add_icon_paths = NULL;
-
-	g_free (dialog->icon_path);
-	dialog->icon_path = NULL;
+	g_clear_object (&(dialog->icon));
 	g_free (dialog->desktop_path);
 	dialog->desktop_path = NULL;
 	g_free (dialog->item_name);
 	dialog->item_name = NULL;
 
-	if (dialog->add_icons_idle_id)
-		g_source_remove (dialog->add_icons_idle_id);
-	dialog->add_icons_idle_id = 0;
-
 	if (dialog->add_items_idle_id)
 		g_source_remove (dialog->add_items_idle_id);
 	dialog->add_items_idle_id = 0;
@@ -238,6 +226,16 @@ panel_run_dialog_destroy (PanelRunDialog *dialog)
 	g_free (dialog);
 }
 
+static const char *
+panel_run_dialog_get_combo_text (PanelRunDialog *dialog)
+{
+	GtkWidget *entry;
+
+	entry = gtk_bin_get_child (GTK_BIN (dialog->combobox));
+
+	return gtk_entry_get_text (GTK_ENTRY (entry));
+}
+
 static void
 panel_run_dialog_set_default_icon (PanelRunDialog *dialog, gboolean set_drag)
 {
@@ -255,39 +253,34 @@ panel_run_dialog_set_default_icon (PanelRunDialog *dialog, gboolean set_drag)
 
 static void
 panel_run_dialog_set_icon (PanelRunDialog *dialog,
-			   const char     *icon_path,
+			   GIcon          *icon,
 			   gboolean        force)
 {
 	GdkPixbuf *pixbuf = NULL;
 
-	if (!force && icon_path && dialog->icon_path &&
-	    !strcmp (icon_path, dialog->icon_path))
+	if (!force && g_icon_equal(icon, dialog->icon))
 		return;
 
-	g_free (dialog->icon_path);
-	dialog->icon_path = NULL;
+	g_clear_object(&(dialog->icon));
 
-	if (icon_path) {
-		GdkScreen   *screen;
-		GtkSettings *settings;
+	if (icon) {
 		int          size;
 
-		screen = gtk_widget_get_screen (GTK_WIDGET (dialog->pixmap));
-		settings = gtk_settings_get_for_screen (screen);
-		gtk_icon_size_lookup_for_settings (settings,
-						   GTK_ICON_SIZE_DIALOG,
-						   &size, NULL);
+		gtk_icon_size_lookup (GTK_ICON_SIZE_DIALOG, &size, NULL);
+
+		GtkIconTheme *icon_theme = gtk_icon_theme_get_default ();
+		GtkIconInfo *icon_info = gtk_icon_theme_lookup_by_gicon (icon_theme, icon, size, 0);
+		pixbuf = gtk_icon_info_load_icon (icon_info, NULL);
+#if GTK_CHECK_VERSION (3, 8, 0)
+		g_object_unref (icon_info);
+#else
+		gtk_icon_info_free (icon_info);
+#endif
 
-		pixbuf = panel_load_icon (gtk_icon_theme_get_default (),
-					  icon_path,
-					  size,
-					  size,
-					  size,
-					  NULL);
 	}
 
 	if (pixbuf) {
-		dialog->icon_path = g_strdup (icon_path);
+		dialog->icon = g_object_ref (icon);
 
 		/* Don't bother scaling the image if it's too small.
 		 * Scaled looks worse than a smaller image.
@@ -298,7 +291,11 @@ panel_run_dialog_set_icon (PanelRunDialog *dialog,
 		//(ditto for the drag icon?)
 		gtk_window_set_icon (GTK_WINDOW (dialog->run_dialog), pixbuf);
 
+#if GTK_CHECK_VERSION (3, 2, 0)
+		gtk_drag_source_set_icon_gicon (dialog->run_dialog, dialog->icon);
+#else
 		gtk_drag_source_set_icon_pixbuf (dialog->run_dialog, pixbuf);
+#endif
 		g_object_unref (pixbuf);
 	} else {
 		panel_run_dialog_set_default_icon (dialog, TRUE);
@@ -349,6 +346,15 @@ command_is_executable (const char   *command,
 	return TRUE;
 }
 
+/*
+ * Set the DISPLAY variable, to be use by g_spawn_async.
+ */
+static void
+set_environment (gpointer display)
+{
+  g_setenv ("DISPLAY", display, TRUE);
+}
+
 static void
 dummy_child_watch (GPid         pid,
 		   		   gint         status,
@@ -360,17 +366,6 @@ dummy_child_watch (GPid         pid,
 	 */
 }
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-/*
- * Set the DISPLAY variable, to be use by g_spawn_async.
- */
-static void
-set_environment (gpointer display)
-{
-	g_setenv ("DISPLAY", display, TRUE);
-}
-#endif
-
 static gboolean
 panel_run_dialog_launch_command (PanelRunDialog *dialog,
 				 const char     *command,
@@ -381,10 +376,8 @@ panel_run_dialog_launch_command (PanelRunDialog *dialog,
 	GError     *error = NULL;
 	char      **argv;
 	int         argc;
-	GPid        pid;
-#if GTK_CHECK_VERSION (3, 0, 0)
 	char       *display;
-#endif
+	GPid        pid;
 
 	if (!command_is_executable (locale_command, &argc, &argv))
 		return FALSE;
@@ -394,31 +387,17 @@ panel_run_dialog_launch_command (PanelRunDialog *dialog,
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->terminal_checkbox)))
 		mate_desktop_prepend_terminal_to_vector (&argc, &argv);
 
-#if GTK_CHECK_VERSION (3, 0, 0)
 	display = gdk_screen_make_display_name (screen);
 
 	result = g_spawn_async (NULL, /* working directory */
 				argv,
 				NULL, /* envp */
-				G_SPAWN_SEARCH_PATH,
+				G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
 				set_environment,
-				&display,
-				NULL,
+				display,
+				&pid,
 				&error);
 
-	g_free (display);
-#else
-	result = gdk_spawn_on_screen (screen,
-				      NULL, /* working directory */
-				      argv,
-				      NULL, /* envp */
-				      G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
-				      NULL, /* child setup func */
-				      NULL, /* user data */
-				      &pid, /* child pid */
-				      &error);
-#endif
-
 	if (!result) {
 		char *primary;
 
@@ -448,7 +427,7 @@ panel_run_dialog_execute (PanelRunDialog *dialog)
 	char     *disk;
 	char     *scheme;
 
-	command = g_strdup (gtk_entry_get_text (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (dialog->combobox)))));
+	command = g_strdup (panel_run_dialog_get_combo_text (dialog));
 	command = g_strchug (command);
 
 	if (!command || !command [0]) {
@@ -643,13 +622,13 @@ fuzzy_command_match (const char *cmd1,
 
 	/* same for the user command */
 	tokens = g_strsplit (cmd2, " ", -1);
-	word2 = g_path_get_basename (tokens [0]);
 	if (!tokens || !tokens [0]) {
 		g_free (word1);
 		g_strfreev (tokens);
 		return FALSE;
 	}
 
+	word2 = g_path_get_basename (tokens [0]);
 	g_strfreev (tokens);
 
 	if (!strcmp (word1, word2)) {
@@ -685,7 +664,7 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 	GtkTreeModel *model;
 	GtkTreePath  *path;
 	char         *text;
-	char         *found_icon;
+	GIcon        *found_icon;
 	char         *found_name;
 	gboolean      fuzzy;
 
@@ -702,30 +681,30 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 		return FALSE;
 	}
 
-	text = g_strdup (gtk_entry_get_text (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (dialog->combobox)))));
+	text = g_strdup (panel_run_dialog_get_combo_text (dialog));
 	found_icon = NULL;
 	found_name = NULL;
 	fuzzy = FALSE;
 
 	do {
 		char *exec = NULL;
-		char *icon = NULL;
+		GIcon *icon = NULL;
 		char *name = NULL;
 		char *comment = NULL;
 
 		gtk_tree_model_get (model, &iter,
 				    COLUMN_EXEC,      &exec,
-				    COLUMN_ICON_FILE, &icon,
+				    COLUMN_GICON,     &icon,
 				    COLUMN_NAME,      &name,
 				    COLUMN_COMMENT,   &comment,
 				    -1);
 
 		if (!fuzzy && exec && icon &&
 		    fuzzy_command_match (text, exec, &fuzzy)) {
-			g_free (found_icon);
+			g_clear_object (&found_icon);
 			g_free (found_name);
 
-			found_icon = g_strdup (icon);
+			found_icon = g_object_ref (icon);
 			found_name = g_strdup (name);
 
 			gtk_list_store_set (dialog->program_list_store,
@@ -747,7 +726,7 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 		}
 
 		g_free (exec);
-		g_free (icon);
+		g_object_unref (icon);
 		g_free (name);
 		g_free (comment);
 
@@ -763,7 +742,7 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 	panel_run_dialog_set_icon (dialog, found_icon, FALSE);
 	//FIXME update dialog->program_label
 
-	g_free (found_icon);
+	g_clear_object (&found_icon);
 	g_free (text);
 
 	g_free (dialog->item_name);
@@ -773,56 +752,6 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 	return FALSE;
 }
 
-static gboolean
-panel_run_dialog_add_icon_idle (PanelRunDialog *dialog)
-{
-	GtkTreeIter  iter;
-	GtkTreePath *path;
-	GdkPixbuf   *pixbuf;
-	char        *file;
-	int          icon_height;
-	gboolean     long_operation = FALSE;
-
-	do {
-		if (!dialog->add_icon_paths) {
-			dialog->add_icons_idle_id = 0;
-			return FALSE;
-		}
-
-		path = dialog->add_icon_paths->data;
-		dialog->add_icon_paths->data = NULL;
-		dialog->add_icon_paths = g_slist_delete_link (dialog->add_icon_paths,
-						              dialog->add_icon_paths);
-
-		if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (dialog->program_list_store),
-					      &iter,
-					      path)) {
-			gtk_tree_path_free (path);
-			continue;
-		}
-
-		gtk_tree_path_free (path);
-
-		gtk_tree_model_get (GTK_TREE_MODEL (dialog->program_list_store), &iter,
-				    COLUMN_ICON_FILE, &file, -1);
-
-		if (!gtk_icon_size_lookup (panel_menu_icon_get_size (), NULL, &icon_height)) {
-			icon_height = PANEL_DEFAULT_MENU_ICON_SIZE;
-		}
-
-		pixbuf = panel_make_menu_icon (gtk_icon_theme_get_default (), file, NULL, icon_height, &long_operation);
-		if (pixbuf) {
-			gtk_list_store_set (dialog->program_list_store, &iter, COLUMN_ICON, pixbuf, -1);
-			g_object_unref (pixbuf);
-		}
-		g_free (file);
-
-	/* don't go back into the main loop if this wasn't very hard to do */
-	} while (!long_operation);
-
-	return TRUE;
-}
-
 static int
 compare_applications (MateMenuTreeEntry *a,
 		      MateMenuTreeEntry *b)
@@ -930,8 +859,7 @@ panel_run_dialog_add_items_idle (PanelRunDialog *dialog)
 
 	/* create list store */
 	dialog->program_list_store = gtk_list_store_new (NUM_COLUMNS,
-							 GDK_TYPE_PIXBUF,
-							 G_TYPE_STRING,
+							 G_TYPE_ICON,
 							 G_TYPE_STRING,
 							 G_TYPE_STRING,
 							 G_TYPE_STRING,
@@ -961,12 +889,12 @@ panel_run_dialog_add_items_idle (PanelRunDialog *dialog)
 	for (l = all_applications; l; l = l->next) {
 		MateMenuTreeEntry *entry = l->data;
 		GtkTreeIter    iter;
-		GtkTreePath   *path;
+		const gchar *icon = matemenu_tree_entry_get_icon (entry);
+		GIcon *gicon = panel_gicon_from_icon_name (icon);
 
 		gtk_list_store_append (dialog->program_list_store, &iter);
 		gtk_list_store_set (dialog->program_list_store, &iter,
-				    COLUMN_ICON,      NULL,
-				    COLUMN_ICON_FILE, matemenu_tree_entry_get_icon (entry),
+				    COLUMN_GICON,     gicon,
 				    COLUMN_NAME,      matemenu_tree_entry_get_display_name (entry),
 				    COLUMN_COMMENT,   matemenu_tree_entry_get_comment (entry),
 				    COLUMN_EXEC,      matemenu_tree_entry_get_exec (entry),
@@ -974,9 +902,7 @@ panel_run_dialog_add_items_idle (PanelRunDialog *dialog)
 				    COLUMN_VISIBLE,   TRUE,
 				    -1);
 
-		path = gtk_tree_model_get_path (GTK_TREE_MODEL (dialog->program_list_store), &iter);
-		if (path != NULL)
-			dialog->add_icon_paths = g_slist_prepend (dialog->add_icon_paths, path);
+		g_object_unref (gicon);
 
 		matemenu_tree_item_unref (entry);
 	}
@@ -994,10 +920,11 @@ panel_run_dialog_add_items_idle (PanelRunDialog *dialog)
 					 COLUMN_NAME);
 
 	renderer = gtk_cell_renderer_pixbuf_new ();
+	g_object_set (renderer, "stock-size", panel_menu_icon_get_size(), NULL);
 	column = gtk_tree_view_column_new ();
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
 	gtk_tree_view_column_set_attributes (column, renderer,
-                                             "pixbuf", COLUMN_ICON,
+                                             "gicon", COLUMN_GICON,
                                              NULL);
 
 	renderer = gtk_cell_renderer_text_new ();
@@ -1008,15 +935,8 @@ panel_run_dialog_add_items_idle (PanelRunDialog *dialog)
 
 	gtk_tree_view_append_column (GTK_TREE_VIEW (dialog->program_list), column);
 
-	dialog->add_icon_paths = g_slist_reverse (dialog->add_icon_paths);
-
-	if (!dialog->add_icons_idle_id)
-		dialog->add_icons_idle_id =
-			g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc) panel_run_dialog_add_icon_idle,
-					 dialog, NULL);
-
 	dialog->add_items_idle_id = 0;
-	return FALSE;
+	return G_SOURCE_REMOVE;
 }
 
 static char *
@@ -1122,7 +1042,9 @@ program_list_selection_changed (GtkTreeSelection *selection,
 	g_free (temp);
 
 	temp = panel_key_file_get_locale_string (key_file, "Icon");
-	panel_run_dialog_set_icon (dialog, temp, FALSE);
+	GIcon *icon = panel_gicon_from_icon_name (temp);
+	panel_run_dialog_set_icon (dialog, icon, FALSE);
+	g_object_unref (icon);
 	g_free (temp);
 
 	temp = panel_key_file_get_locale_string (key_file, "Comment");
@@ -1168,6 +1090,8 @@ panel_run_dialog_setup_program_list (PanelRunDialog *dialog,
 	dialog->program_label = PANEL_GTK_BUILDER_GET (gui, "program_label");
 	dialog->main_box = PANEL_GTK_BUILDER_GET (gui, "main_box");
 
+	gtk_widget_set_can_focus (dialog->program_label, FALSE);
+
 	/* Ref the box so it doesn't get destroyed when it is
 	 * removed from the visible area of the dialog box.
 	 */
@@ -1633,7 +1557,7 @@ combobox_changed (GtkComboBox    *combobox,
 	char *start;
 	char *msg;
 
-	text = g_strdup (gtk_entry_get_text (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (combobox)))));
+	text = g_strdup (panel_run_dialog_get_combo_text (dialog));
 
 	start = text;
 	while (*start != '\0' && g_ascii_isspace (*start))
@@ -1818,7 +1742,7 @@ panel_run_dialog_create_desktop_file (PanelRunDialog *dialog)
 	char     *scheme;
 	char     *save_uri;
 
-	text = g_strdup (gtk_entry_get_text (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (dialog->combobox)))));
+	text = g_strdup (panel_run_dialog_get_combo_text (dialog));
 
 	if (!text || !text [0]) {
 		g_free (text);
@@ -1860,9 +1784,11 @@ panel_run_dialog_create_desktop_file (PanelRunDialog *dialog)
 	panel_key_file_set_boolean (key_file, "Terminal",
 				    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->terminal_checkbox)));
 
-	if (dialog->icon_path)
-		panel_key_file_set_locale_string (key_file, "Icon",
-						  dialog->icon_path);
+	if (dialog->icon) {
+		gchar *icon_path = g_icon_to_string (dialog->icon);
+		panel_key_file_set_locale_string (key_file, "Icon", icon_path);
+		g_free (icon_path);
+	}
 	else
 		panel_key_file_set_locale_string (key_file, "Icon",
 						  PANEL_ICON_LAUNCHER);
@@ -1907,16 +1833,21 @@ pixmap_drag_data_get (GtkWidget          *run_dialog,
 }
 
 static void
+#if GTK_CHECK_VERSION (3, 0, 0)
+panel_run_dialog_style_updated (GtkWidget *widget,
+				PanelRunDialog *dialog)
+#else
 panel_run_dialog_style_set (GtkWidget      *widget,
 			    GtkStyle       *prev_style,
 			    PanelRunDialog *dialog)
+#endif
 {
-  if (dialog->icon_path) {
-	  char *icon_path;
+  if (dialog->icon) {
+	  GIcon *icon;
 
-	  icon_path = g_strdup (dialog->icon_path);
-	  panel_run_dialog_set_icon (dialog, icon_path, TRUE);
-	  g_free (icon_path);
+	  icon = g_object_ref (dialog->icon);
+	  panel_run_dialog_set_icon (dialog, icon, TRUE);
+	  g_object_unref (icon);
   }
 }
 
@@ -1925,12 +1856,12 @@ panel_run_dialog_screen_changed (GtkWidget      *widget,
 				 GdkScreen      *prev_screen,
 				 PanelRunDialog *dialog)
 {
-  if (dialog->icon_path) {
-	  char *icon_path;
+  if (dialog->icon) {
+	  GIcon *icon;
 
-	  icon_path = g_strdup (dialog->icon_path);
-	  panel_run_dialog_set_icon (dialog, icon_path, TRUE);
-	  g_free (icon_path);
+	  icon = g_object_ref (dialog->icon);
+	  panel_run_dialog_set_icon (dialog, icon, TRUE);
+	  g_object_unref (icon);;
   }
 }
 
@@ -1940,9 +1871,15 @@ panel_run_dialog_setup_pixmap (PanelRunDialog *dialog,
 {
 	dialog->pixmap = PANEL_GTK_BUILDER_GET (gui, "icon_pixmap");
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	g_signal_connect (dialog->pixmap, "style-updated",
+			  G_CALLBACK (panel_run_dialog_style_updated),
+			  dialog);
+#else
 	g_signal_connect (dialog->pixmap, "style-set",
 			  G_CALLBACK (panel_run_dialog_style_set),
 			  dialog);
+#endif
 	g_signal_connect (dialog->pixmap, "screen-changed",
 			  G_CALLBACK (panel_run_dialog_screen_changed),
 			  dialog);
@@ -2003,9 +1940,15 @@ panel_run_dialog_new (GdkScreen  *screen,
 static void
 panel_run_dialog_disconnect_pixmap (PanelRunDialog *dialog)
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	g_signal_handlers_disconnect_by_func (dialog->pixmap,
+					      G_CALLBACK (panel_run_dialog_style_updated),
+					      dialog);
+#else
 	g_signal_handlers_disconnect_by_func (dialog->pixmap,
 			                      G_CALLBACK (panel_run_dialog_style_set),
 			                      dialog);
+#endif
 	g_signal_handlers_disconnect_by_func (dialog->pixmap,
 			                      G_CALLBACK (panel_run_dialog_screen_changed),
 			                      dialog);
