@@ -39,60 +39,6 @@ static Atom net_wm_window_type_normal = None;
 static Atom net_wm_strut              = None;
 static Atom net_wm_strut_partial      = None;
 
-void
-panel_xutils_set_window_type (GdkWindow             *gdk_window,	
-			      PanelXUtilsWindowType  type)
-{
-	Display *display;
-	Window   window;
-	Atom     atoms [2];
-	int      i = 0;
-
-	g_return_if_fail (GDK_IS_WINDOW (gdk_window));
-
-	display = GDK_WINDOW_XDISPLAY (gdk_window);
-#if GTK_CHECK_VERSION (3, 0, 0)
-	window = GDK_WINDOW_XID (gdk_window);
-#else
-	window  = GDK_WINDOW_XWINDOW (gdk_window);
-#endif
-
-	if (net_wm_window_type == None)
-		net_wm_window_type = XInternAtom (display,
-						  "_NET_WM_WINDOW_TYPE",
-						  False);
-
-	switch (type) {
-	case PANEL_XUTILS_TYPE_DOCK:
-		if (net_wm_window_type_dock == None)
-			net_wm_window_type_dock = XInternAtom (display,
-							       "_NET_WM_WINDOW_TYPE_DOCK",
-							       False);
-		atoms [i++] = net_wm_window_type_dock;
-		break;
-	case PANEL_XUTILS_TYPE_NORMAL:
-		if (net_wm_window_type_normal == None)
-			net_wm_window_type_normal = XInternAtom (display,
-								 "_NET_WM_WINDOW_TYPE_NORMAL",
-								 False);
-		atoms [i++] = net_wm_window_type_normal;
-		break;
-	default:
-		g_assert_not_reached ();
-		break;
-	}
-
-	gdk_error_trap_push ();
-	XChangeProperty (display, window, net_wm_window_type,
-			 XA_ATOM, 32, PropModeReplace,
-			 (guchar *) &atoms, i);
-#if GTK_CHECK_VERSION (3, 0, 0)
-	gdk_error_trap_pop_ignored ();
-#else
-	gdk_error_trap_pop ();
-#endif
-}
-
 enum {
 	STRUT_LEFT = 0,
 	STRUT_RIGHT = 1,
@@ -122,11 +68,7 @@ panel_xutils_set_strut (GdkWindow        *gdk_window,
 	g_return_if_fail (GDK_IS_WINDOW (gdk_window));
 
 	display = GDK_WINDOW_XDISPLAY (gdk_window);
-#if GTK_CHECK_VERSION (3, 0, 0)
 	window = GDK_WINDOW_XID (gdk_window);
-#else
-	window  = GDK_WINDOW_XWINDOW (gdk_window);
-#endif
 
 	if (net_wm_strut == None)
 		net_wm_strut = XInternAtom (display, "_NET_WM_STRUT", False);
@@ -181,11 +123,7 @@ panel_warp_pointer (GdkWindow *gdk_window,
 	g_return_if_fail (GDK_IS_WINDOW (gdk_window));
 
 	display = GDK_WINDOW_XDISPLAY (gdk_window);
-#if GTK_CHECK_VERSION (3, 0, 0)
 	window = GDK_WINDOW_XID (gdk_window);
-#else
-	window  = GDK_WINDOW_XWINDOW (gdk_window);
-#endif
 
 	gdk_error_trap_push ();
 	XWarpPointer (display, None, window, 0, 0, 0, 0, x, y);
