@@ -21,7 +21,6 @@
 #include <gdk/gdkkeysyms.h>
 #if GTK_CHECK_VERSION (3, 0, 0)
 #include <gtk/gtkx.h> /* for GTK_IS_SOCKET */
-#include <gdk/gdkkeysyms-compat.h>
 #endif
 
 #include <libpanel-util/panel-glib.h>
@@ -398,13 +397,9 @@ panel_key_press_event (GtkWidget   *widget,
 	 * activating the key bindings here.
 	 */ 
 	if (GTK_IS_SOCKET (gtk_window_get_focus (GTK_WINDOW (widget))) &&
-	    event->keyval == GDK_F10 &&
+	    event->keyval == GDK_KEY_F10 &&
 	    (event->state & gtk_accelerator_get_default_mod_mask ()) == GDK_CONTROL_MASK)
-#if GTK_CHECK_VERSION (3, 0, 0)
 		return gtk_bindings_activate (G_OBJECT (widget),
-#else
-		return gtk_bindings_activate (GTK_OBJECT (widget),
-#endif
 					      event->keyval,
 					      event->state);
 
