@@ -309,6 +309,14 @@ zoom_timeout (GtkWidget *window)
 	return TRUE;
 }
 
+static gboolean
+idle_destroy (gpointer data)
+{
+	gtk_widget_destroy (GTK_WIDGET (data));
+
+	return FALSE;
+}
+
 static gboolean
 #if GTK_CHECK_VERSION (3, 0, 0)
 zoom_draw (GtkWidget *widget,
@@ -328,12 +336,13 @@ zoom_expose (GtkWidget      *widget,
 			g_source_remove (zoom->timeout_id);
 		zoom->timeout_id = 0;
 
+		gtk_widget_hide (widget);
+		g_idle_add (idle_destroy, widget);
+
 		g_object_unref (zoom->pixbuf);
 		zoom->pixbuf = NULL;
 
 		g_slice_free (CompositedZoomData, zoom);
-
-		gtk_widget_destroy (widget);
 	} else {
 		GdkPixbuf *scaled;
 		int width, height;
@@ -700,7 +709,11 @@ xstuff_grab_key_on_all_screens (int      keycode,
 	int         i;
 
 	display   = gdk_display_get_default ();
+#if GTK_CHECK_VERSION(3, 10, 0)
+	n_screens = 1; /* gdk-3.10, The number of screens is always 1 */
+#else
 	n_screens = gdk_display_get_n_screens (display);
+#endif
 
 	for (i = 0; i < n_screens; i++) {
 		GdkWindow *root;
